///////////////////////////////////////////////////////////////////////////////
//
//      TargaImage.h                            Author:     Stephen Chenney
//                                              Modified:   Eric McDaniel
//                                              Date:       Spring 2002
//
//      Class to manipulate targa images.  You must implement the image 
//  modification functions.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _TARGA_IMAGE_H_
#define _TARGA_IMAGE_H_

#include <Fl/Fl.h>
#include <Fl/Fl_Widget.h>
#include <stdio.h>

class TargaImage
{
    // methods
    public:
	    TargaImage(void);
	    TargaImage(int, int, unsigned char*);
	    ~TargaImage(void);

        unsigned char*	To_RGB(void);	            // Convert the image to RGB format,
        bool Save_Image(char*);                     // save the image to a file
        static TargaImage* Load_Image(char*);       // Load a file and return a pointer to a new TargaImage object.  Returns NULL on failure

        bool To_Grayscale();

        bool Quant_Uniform();
        bool Quant_Populosity();
        bool Quant_Median();

        bool Dither_Threshold();
        bool Dither_Random();
        bool Dither_Ordered();
        bool Dither_FS();
        bool Dither_Bright();
        bool Dither_Cluster();

        bool Comp_Over(TargaImage* pImage);
        bool Comp_In(TargaImage* pImage);
        bool Comp_Out(TargaImage* pImage);
        bool Comp_Atop(TargaImage* pImage);
        bool Comp_Xor(TargaImage* pImage);

        bool Difference(TargaImage* pImage);

        bool Filter_Box();
        bool Filter_Bartlett();
        bool Filter_Gaussian();
        bool Filter_Edge();
        bool Filter_Enhance();

        bool Half_Size();
        bool Double_Size();
        bool Resize(float scale);
        bool Blue_Screen();
        bool Rotate(float angleDegrees);

    private:
        void RGBA_To_RGB(unsigned char *data, unsigned char *rgb);          // helper function for format conversion
	    TargaImage* Reverse_Rows(void);                                     // reverse the rows of the image, some targas are stored bottom to top
        void ClearToBlack();                                                // clear image to all black
        void Filter_Image(float **filter, int fw, int fh, bool do_alpha);

    // members
    public:
        int		        width;	    // width of the image in pixels
        int		        height;	    // height of the image in pixels
        unsigned char	*data;	    // pixel data for the image, assumed to be in pre-multiplied RGBA format.

};


#endif


